#!/usr/bin/perl -w 

use strict;
use vars qw($opt_h $opt_H $opt_e $opt_F);

require "getopts.pl";		# read and interpret command line arguments
&Getopts('FhHe:');


if ($opt_h) {
    $~ = "HELP";
    write;
    exit 0; }
if ($opt_H) {
    $~ = "HELP";
    write;
    $~ = "LONGHELP";
    write;
    exit 0; }



#holds the column numbers for different data types
my( $e_column);
my( $xmu_column);

my($name);      #file name from command line
my($line);      #read line from data file

my(%DATA);      #DATA line indexed by energy
my(@energy);  #energy array
my($i)=0;  #index for energy array
my($energy,$xmu);

my($Eave,$result);

unless (defined $opt_e){
  print " binpt: What is value for edge energy?\n";
  $~ = "HELP";
  write;
  $~ = "LONGHELP";
  write;
  exit 0; 
}

foreach $name (@ARGV) {
  print "\nbinpt: reading $name\n";
  if (!($name =~ /_/)){
    print "do you really want to run binpt on $name?";
    my($ans) = <STDIN>;
    next if ($ans =~ /[nN]/);
  }
  my($outname) = $name;
  $outname =~ s/([tfre])\./$1b\./;  #Add b to output file name
  if (-e "${outname}" && !($opt_F) ){
    print "${outname} exsists skipping\n";
    next;
  }
  open(OUT,">${outname}") || 
    die "cannot open ${outname}";
  print OUT "#written by binpt.pl. The original file name is $name\n";
  print OUT "#-------------------\n";
  print OUT "# energy\txmu\n";
  open(DATA, $name) || die "cannot open $name";
  while ($line = <DATA>){
    if ($line =~ /[eE]nergy.*xmu/){   #look for headder line
      while ($line = <DATA>){         #read in data after headder
	$line =~ s/^[\s]+|[\s]+$//g;  #remove leading and trailing space
	$line =~ s/[\s]+/ /g;         #replace all other space with single space
	#print "line=$line\n";
	($energy,$xmu) = split(/\s/,$line);  #put data into energy and xmu
	#print "energy=$energy,xmu=$xmu\n";
	$energy[$i] = $energy;               #put energy into an array
	$DATA{$energy} = $xmu;             #put xmu into a hash 
	$i++;
      }
    }
  }
  close(DATA); #close data file
  #for($i=0; $i<@energy; $i++){
  #  print "energy=$energy[$i],xmu=$DATA{$energy[$i]}\n";
  #}
  for ($i=0; $i<@energy; $i++){             #bunch 10eV for 50eV below edge
    if ($energy[$i] < $opt_e - 50){
      ($i,$Eave, $result) = &bunch10eV($i);
      &print_point($Eave,$result);
    }elsif($energy[$i] < $opt_e + 50){     #bunch 0.5eV for edge region
      ($i,$Eave,$result) = &bunch05eV($i);
      &print_point($Eave,$result);
    }elsif($energy[$i] > $opt_e +50){      #bunch 0.005 1/angstrom for the rest
      ($i,$Eave,$result) = &bunch0005k($i);
      &print_point($Eave,$result);
    }
  }
  close(OUT);
  print "\twrote  $outname\n";
  &initialize_parameters;
}



sub bunch10eV{
  my($index) = $i;
  my($Etotal,$xmutotal) = 0;
  my($num) = 0;
  my($initialE) = $energy[$index];
  while($energy[$index] < $initialE + 10){
    $num++;
    $Etotal += $energy[$index];
    $xmutotal += $DATA{$energy[$index]};
    $index++;
  }
  $Eave = $Etotal/$num;
  $result = $xmutotal/$num;
  return ($index-1,$Eave,$result);
}
sub bunch05eV{
  my($index) = $i;
  my($Etotal,$xmutotal) = 0;
  my($num) = 0;
  my($initialE) = $energy[$index];
  while($energy[$index] < $initialE + 0.5){
    $num++;
    $Etotal += $energy[$index];
    $xmutotal += $DATA{$energy[$index]};
    $index++;
  }
  $Eave = $Etotal/$num;
  $result = $xmutotal/$num;
  return ($index-1,$Eave,$result);
}
sub bunch0005k{
  my($index) = $i;
  my($Etotal,$xmutotal) = 0;
  my($num) = 0;
  my($initialk) =  sqrt($energy[$index]/3.81);
  #print "\tinitialk = $initialk\n";
  my($k) =  sqrt($energy[$index]/3.81);
  while($k < $initialk + 0.005){
    $num++;
    $Etotal += $energy[$index];
    $xmutotal += $DATA{$energy[$index]};
    $index++;
    last,if(!(defined($energy[$index])));
    $k =  sqrt($energy[$index]/3.81);
    #print "\t next k=$k\n";
  }
  $Eave = $Etotal/$num;
  #print "Eave = $Eave\n";
  $result = $xmutotal/$num;
  return ($index-1,$Eave,$result);
}
  


sub initialize_parameters{
  my $key;
  $i = 0;
  my($index);
  foreach $index (@energy){
    delete($energy[$index]);}
  foreach $key (keys %DATA){
    delete($DATA{$key})
  }
  $Eave = 0;
  $result = 0;
}


sub print_point{
  my($e, $result) = @_;
  printf OUT  "  %9.7E   %9.7E\n", $e,$result;
}



format HELP = 
 binpt (13 December 2002) bins data points for quickscans collected at MRCAT
 usage :    binpt.pl -e #  filename(s)
..

format LONGHELP =

  This is a script that averages points to reduce the
  size of the data files.
  Use -e # to specify edge energy
  Data points from begining of data file to E-50 are
  binned in 10eV step size.
  Data points from E-50 to E+50 are
  binned in 0.5eV step size.
  Data points from E+50 to the end of data file are
  binned in 0.005 1/Angstrom step size.

..
